--
-- lust
--
-- Copyright (c) 2014, rnlf
--
-- This library is free software; you can redistribute it and/or modify it
-- under the terms of the MIT license. See LICENSE for details.
--

--- Very minimalistic camera class
local Camera = Object:extend()

--- Create a new Camera. Call Camera(...) to call and obtain the new instance
--  @param w The width of the viewport
--  @param g The height of the viewport
--  @param cx The x coordinate of the viewport's center
--  @param cy The y coordinate of the viewport's center
function Camera:new(w,h,cx,cy)
  self.x = cx
  self.y = cy
  self.w = w
  self.h = h
  self.tween = flux.group()
  self.shakeAmount = 0.0
end

--- Set up love's transformation matrix to use this camera
function Camera:setup()
  love.graphics.origin()
  local sa = self.shakeAmount
  local r = love.math.random
  local f = math.floor
  love.graphics.translate(f(r() * sa + self.w / 2 - self.x),
                          f(r() * sa + self.h / 2 - self.y))
  love.graphics.setScissor(0,0, self.w, self.h)
end

function Camera:update(dt)
  self.tween:update(dt)
end

function Camera:shake(amount)
  self.shakeAmount = amount
end


function Camera:shakeTween(amount, time)
  self.tween:to(self, time or 1, {shakeAmount = amount})
end


function Camera:shakeDecay(amount, time)
  self.shakeAmount = amount
  self.tween:to(self, time or 1, {shakeAmount = 0})
end

return Camera
